﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using MbUnit.Framework;
using System.Diagnostics;
using NHibernate.Validator.Engine;
using System.Collections;
using NHibernate.Mapping;
using NHibernate.Validator.Constraints;

namespace ClassLibrary1
{
    class Tests
    {
        [FixtureSetUp]
        public void _TestFixtureSetUp()
        {
            NHibernate.Validator.Cfg.Environment.SharedEngineProvider =
               new NHibernate.Validator.Event.NHibernateSharedEngineProvider();
        }

        [Test]
        public void Can_Create_Database_from_Mapping_Files()
        {
            NHibernate.Cfg.Configuration cfg = new NHibernate.Cfg.Configuration();

            cfg.Configure();

            NHibernate.Tool.hbm2ddl.SchemaExport schemaExport = new NHibernate.Tool.hbm2ddl.SchemaExport(cfg);

            schemaExport.Create(true, false);
        }

        [Test]
        public void Can_Create_Database_from_Mapping_Files_With_Validator()
        {
            NHibernate.Cfg.Configuration cfg = new NHibernate.Cfg.Configuration();

            cfg.Configure();

            var engine = new ValidatorEngine();
            engine.Configure(new NHibernate.Validator.Cfg.NHVConfigurationBase());
            NHibernate.Validator.Cfg.ValidatorInitializer.Initialize(cfg, engine);

            NHibernate.Tool.hbm2ddl.SchemaExport schemaExport =
                new NHibernate.Tool.hbm2ddl.SchemaExport(cfg);

            schemaExport.Create(true, false);
        }

        [Test]
        public void Customer_Age_can_be_validated_ALONE()
        {
            var validator = NHibernate.Validator.Cfg.Environment.SharedEngineProvider.GetEngine();

            Customer customer = new Customer() { Firstname = null, Lastname = null, Age = 7 };

            Assert.IsFalse(customer.IsValid());

            var brokenRules = validator.ValidatePropertyValue<Customer, int>(customer, c => c.Age);

            Assert.IsEmpty(brokenRules);
        }

        [Test]
        public void Customer_can_return_messages_when_invalid()
        {

            Customer customer = new Customer() { Firstname = null };

            NHibernate.Validator.Engine.InvalidValue[] brokenRules = customer.Validate();

            foreach (NHibernate.Validator.Engine.InvalidValue invalidValue in brokenRules)
            {
                Debug.WriteLine(String.Format("{0} {1} and you entered {2}", invalidValue.PropertyName, invalidValue.Message, invalidValue.Value));
            }

        }

        [Test]
        public void Customer_Can_Validate_Gender_When_Invalid()
        {
            var validator = NHibernate.Validator.Cfg.Environment.SharedEngineProvider.GetEngine();

            Customer customer = new Customer() { Gender = "Steve" };

            Assert.IsFalse(customer.IsValid());

            var brokenRules = validator.ValidatePropertyValue<Customer, string>(customer, c => c.Gender);

            Assert.IsNotEmpty(brokenRules);

            foreach (InvalidValue invalidValue in brokenRules)
            {
                Debug.WriteLine(invalidValue.Message);
            }

        }

        [Test]
        public void Customer_Cannot_Be_Persisted_If_Not_Valid()
        {
            NHibernate.Cfg.Configuration cfg = new NHibernate.Cfg.Configuration();

            cfg.Configure();

            NHibernate.ISessionFactory sessionFactory = cfg.BuildSessionFactory();

            Customer invalidCustomer = new Customer()
            {
                Firstname = null, Lastname = null, Age = -3, Gender = "Steve"
            };

            try
            {
                using (var session = sessionFactory.OpenSession())
                {
                    using (var tx = session.BeginTransaction())
                    {
                        session.SaveOrUpdate(invalidCustomer);
                        tx.Commit();
                    }
                }

                Assert.Fail("We should never get to this line of code!!!");
            }

            catch (NHibernate.Validator.Exceptions.InvalidStateException ex)
            {
                foreach (InvalidValue invalidValue in ex.GetInvalidValues())
                {
                    Debug.WriteLine(String.Format("{0} {1}", invalidValue.PropertyName, invalidValue.Message));
                }
            }
        }

        [Test]
        public void Customer_is_valid()
        {

            var validator = NHibernate.Validator.Cfg.Environment.SharedEngineProvider.GetEngine();

            Customer customer = new Customer() { Firstname = "Steve", Lastname = "Bohlen", Age = 16, Gender = "male" };

            Assert.IsTrue(validator.IsValid(customer));

            Order order = new Order() { Ordernumber = 1 };

            Assert.IsFalse(validator.IsValid(order));

            Item item = new Item() { Quantity = 0 };

            Assert.IsFalse(validator.IsValid(item));

            order.Items.Add(item);

            item.Quantity = 1;

            Assert.IsTrue(validator.IsValid(item));

            customer.Orders.Add(order);

            Assert.IsTrue(validator.IsValid(customer));

        }

    }

}
